#ifndef DOGM_SPI_H_
#define DOGM_SPI_H_

#include <msp430x20x3.h>
#include <stdint.h>

#define SPIRS 0x80    /* Hier die Bitmasken fr die verwendeten Ports einstellen! */
#define SPISDA 0x40   /* z.B. RS an P1.7 ist SPIRS 0x80 (binr 1000 0000) */
#define SPISCL 0x20   /* und CSB an P1.0 ist SPICSB 0x1 (binr 0000 0001) */
#define SPICSB 0x01

#define SET1(flags) P1OUT |= (flags); /* Mit diesen Makros die Bits von oben stellen! */
#define RST1(flags) P1OUT &= ~(flags);

#define T30us 1
#define T2ms 186
#define T50ms 10000
#define T200ms 20000  /* Zeitkonstanten fr die sleep()-Funktion: */

void sleep(long time) { /* siehe oben */
	for(;time > 0; time--);	
}

void lcdInit() { /* Bereitet die Ports (s.o.), DCO und USI vor. */

  P1OUT &= ~(SPIRS + SPISDA + SPISCL + SPICSB);
  P1DIR |=   SPIRS + SPISDA + SPISCL + SPICSB;
	
  BCSCTL1 = CALBC1_1MHZ;               // Set DCO
  DCOCTL = CALDCO_1MHZ;	

  USICTL0 |= USIPE6 + USIPE5 + USIMST + USIOE;// Port, SPI master, MSB first
  USICKCTL = USIDIV_4 + USISSEL_2;
  USICTL0 &= ~USISWRST;                     // USI released for operation

  USISR = 0x00;
  USICNT = 1;
}

void lcdDo(unsigned char command) { /* Fhrt ein Kommando aus (IR) */
  RST1(SPIRS); // Instruction Register
  RST1(SPICSB); // !CSB Enable

  USISRL = command;
  USICNT = 8; // send command
  
  while (!(USICTL1 & USIIFG)); // wait for usi

  SET1(SPICSB); // !CSB Disable
}
void lcdWrite(unsigned char data) { /* Schreibt Daten in den Textpuffer (DR) */
  SET1(SPIRS); // Data Register
  RST1(SPICSB); // !CSB Enable

  USISRL = data;
  USICNT = 8; // send command
  
  while (!(USICTL1 & USIIFG)); // wait for usi

  SET1(SPICSB); // !CSB Disable
}

void lcdBoot163Default5V() { /* Optimierter Kontrast fr DOGM 163 an 5V */
  sleep(T50ms); // get up after power on
  lcdDo(38); sleep(T2ms); // Function Set (8bit, 2Lines, no DblHeight, IS[0:0])
  
  lcdDo(0x39); sleep(T30us); // Function Set (8bit, 2Lines, no DblHeight, IS[0:0])
  lcdDo(0x1D); sleep(T30us); // Bias Set (1/4bias, 3Lines)
  lcdDo(0x50); sleep(T30us); // Prw/Icn/Contr (Icon off, Boost off, C5,4 = 0,0)
  lcdDo(0x6C); sleep(T30us); // Follower Control (On, AmpRatio 100)
  lcdDo(0x76); sleep(T30us); // Contrast Set (C3,2,1,0 = 0,1,1,0)
  lcdDo(0x38); sleep(T30us); // Function Set (8bit, 2Lines, no DblHeight, IS[0:0])
  lcdDo(0x0F); sleep(T30us); // Display (Disp on, Cur on, CPos on)
  lcdDo(0x01); sleep(T2ms); // Clear Display
  lcdDo(0x06); sleep(T30us); // Entry Mode Set (Cur Auto-Increment)
}

#endif /*DOGM_SPI_H_*/

